import logging

from pytorch_lightning.callbacks.base import Callback

from graphsignal.proto import profiles_pb2
from graphsignal.profilers.pytorch import PyTorchProfiler
from graphsignal.profiling_span import ProfilingSpan

logger = logging.getLogger('graphsignal')


class GraphsignalCallback(Callback):
    __slots__ = [
        '_profiler',
        '_span'
    ]

    def __init__(self):
        self._profiler = PyTorchProfiler()
        self._span = None
        self._action_name = None
        super().__init__()

    def _start_profiler(self, span_name, span_type, batch=None):
        if not self._span:
            self._span = ProfilingSpan(
                framework_profiler=self._profiler,
                span_name=span_name,
                span_type=span_type)
            if batch is not None:
                self._span.add_metadata('batch', batch)

    def _stop_profiler(self):
        if self._span:
            self._span.stop()
            self._span = None

    def on_train_batch_start(self, trainer, pl_module, batch, batch_idx):
        self._start_profiler(
            'Training batch', profiles_pb2.Span.SpanType.TRAINING_BATCH, batch=batch_idx)

    def on_train_batch_end(self, trainer, pl_module, outputs, batch, batch_idx):
        self._stop_profiler()

    def on_validation_batch_start(self, trainer, pl_module, batch, batch_idx, dataloader_idx):
        self._start_profiler(
            'Validation batch', profiles_pb2.Span.SpanType.VALIDATION_BATCH, batch=batch_idx)

    def on_validation_batch_end(self, trainer, pl_module, outputs, batch, batch_idx, dataloader_idx):
        self._stop_profiler()

    def on_test_batch_start(self, trainer, pl_module, batch, batch_idx, dataloader_idx):
        self._start_profiler(
            'Test batch', profiles_pb2.Span.SpanType.TEST_BATCH, batch=batch_idx)

    def on_test_batch_end(self, trainer, pl_module, outputs, batch, batch_idx, dataloader_idx):
        self._stop_profiler()



import logging
from transformers import TrainerCallback

from graphsignal.proto import profiles_pb2
from graphsignal.profiling_span import ProfilingSpan

logger = logging.getLogger('graphsignal')


class GraphsignalPTCallback(TrainerCallback):
    __slots__ = [
        '_profiler',
        '_span'
    ]

    def __init__(self):
        from graphsignal.profilers.pytorch import PyTorchProfiler
        self._profiler = PyTorchProfiler()
        self._span = None

    def _start_profiler(self, span_name, span_type, args, state):
        if not self._span:
            self._span = ProfilingSpan(
                framework_profiler=self._profiler,
                span_name=span_name,
                span_type=span_type)
            if args:
                if args.run_name:
                    self._span.add_metadata('Run name', args.run_name)
            if state:
                if state.epoch is not None:
                    self._span.add_metadata('Epoch', state.epoch)
                if state.global_step is not None:
                    self._span.add_metadata('Global step', state.global_step)

    def _stop_profiler(self):
        if self._span:
            self._span.stop()
            self._span = None

    def on_step_begin(self, args, state, control, **kwarg):
        self._start_profiler(
            'Training step', profiles_pb2.Span.SpanType.TRAINING_STEP, args, state)

    def on_step_end(self, args, state, control, **kwarg):
        self._stop_profiler()


class GraphsignalTFCallback(TrainerCallback):
    __slots__ = [
        '_profiler',
        '_span'
    ]

    def __init__(self):
        from graphsignal.profilers.tensorflow import TensorflowProfiler
        self._profiler = TensorflowProfiler()
        self._span = None

    def _start_profiler(self, span_name, span_type, args, state):
        if not self._span:
            self._span = ProfilingSpan(
                framework_profiler=self._profiler,
                span_name=span_name,
                span_type=span_type)
            if args:
                if args.run_name:
                    self._span.add_metadata('Run name', args.run_name)
            if state:
                if state.epoch is not None:
                    self._span.add_metadata('Epoch', state.epoch)
                if state.global_step is not None:
                    self._span.add_metadata('Global step', state.global_step)


    def _stop_profiler(self):
        if self._span:
            self._span.stop()
            self._span = None

    def on_step_begin(self, args, state, control, **kwarg):
        self._start_profiler(
            'Training step', profiles_pb2.Span.SpanType.TRAINING_STEP, args, state)

    def on_step_end(self, args, state, control, **kwarg):
        self._stop_profiler()


import unittest
import logging
import sys
from unittest.mock import patch, Mock
import torch
from google.protobuf.json_format import MessageToJson
import pprint
import time

import graphsignal
from graphsignal.usage.process_reader import ProcessReader
from graphsignal.proto import profiles_pb2

logger = logging.getLogger('graphsignal')


class ProcessReaderTest(unittest.TestCase):
    def setUp(self):
        if len(logger.handlers) == 0:
            logger.addHandler(logging.StreamHandler(sys.stdout))
        graphsignal.configure(
            api_key='k1',
            workload_name='w1',
            debug_mode=True)

    def tearDown(self):
        graphsignal.shutdown()

    def test_read(self):
        profile = profiles_pb2.MLProfile()
        reader = graphsignal._agent.process_reader
        reader.read(profile)
        ProcessReader.MIN_CPU_READ_INTERVAL = 0
        time.sleep(0.2)
        reader.read(profile)

        #pp = pprint.PrettyPrinter()
        # pp.pprint(MessageToJson(profile))

        self.assertNotEqual(profile.process_usage[0].hostname, '')
        self.assertNotEqual(profile.process_usage[0].process_id, '')
        self.assertTrue(profile.process_usage[0].cpu_usage_percent > 0)
        self.assertTrue(profile.process_usage[0].max_rss > 0)
        self.assertTrue(profile.process_usage[0].current_rss > 0)
        self.assertTrue(profile.process_usage[0].vm_size > 0)
